# Updating to cenode.js v2.

## Migration and update nodes

CENode v2. comes with a series of enhancements that should make programming with the library a bit easier.

The library is now more modular, in the sense that a lot of the work the CENode class used to do is now spread out amongst the new CEInstance and CEConcept classes. For more information on these, please see the API docs in the root of this repository.

### New features

This version introduces the following new features:

#### Helpers

CENode v2 ships with a very large number of new properties that should make programming with the library more intuitive. The following examples demonstrate some of the features behind this update, and also demonstrate the easier use of the CEConept and CEInstance classes. The examples assume a `CENode` instance (called `node`) has already been instantiated.

```javascript
node.agent // the CEAgent of this node

node.instances.mrs_smith // the CEInstance representing Mrs Smith
node.instances.mrs_smith.teaches // the last-reported CEInstance of the class Mrs Smith teaches
node.instances.card_1.is_to // the last-reported CEInstance of the agent/user Card 1 is sent to
node.instances.card_1.is_tos // an array of recipients of Card 1
node.instances.mrs_smith.synonyms // alternative names for Mrs Smith
node.instances.mrs_smith.age // the last-reported age of Mrs Smith
node.instances.mrs_smith.ages // array of all reported ages of Mrs Smith
node.instances.mrs_smith.type // CEConcept of the teacher concept
node.instances.mrs_smith.type.ancestors // CEConcept array of ancestors of teacher concept
node.instances.mrs_smith.type.parents[0].all_instances // CEInstance array of any type of the first parent of the teacher concept

node.concepts.teacher // CEConcept representing a teacher
node.concepts.teacher.instances // list of instances of type teacher
var teacher_concept = node.concepts.teacher // assign this CEConcept
teacher_concept.instances.mrs_smith.name // Mrs Smith
teacher_concept.teaches // CEConcept representing a class
...
```

The list goes on, and the chaining capability (as demonstrated) can be quite powerful for querying the KB. The API documentation lists most of the available helpers, if you'd like further information.

#### Synonyms

See the relevant [ticket](https://github.com/flyingsparx/CENode/issues/10).

CENode v2 supports the assignment of synonyms to concepts and instances. These are defined in CE for both concepts and instances in a similar way.

For concepts:

`conceptualise the teacher T ~ is expressed by ~ 'instructor' and is ...`

And for instances:

`there is a teacher named 'Mrs Smith' that is expressed by 'Jane Smith' and is expressed by 'J Smith'`

Once defined, synonyms can be used interchangeably:

`the instructor 'Jane Smith' teaches the ...`

_Please note_: synonyms must be defined in valid CE, and the node's NL-parser is not (currently) equipped to handle the 'expressed by' clause.

#### Synchronous card-handling

See the relevant [ticket](https://github.com/flyingsparx/CENode/issues/12).

Users of the library writing for more lightweight systems may not have access to key requirements of the library, such as the `window.setTimeout()` function. To help with this problem, CENode now allows developers to synchronously process cards with the agent, using the `agent.handle_card(CEInstance)` function.

Please see the API documentation for further information on this.

#### General

The new modular approach to the library will allow for much easier adaption and growth, allowing new features to be added. By clearly segregating concept objects from instance objects (and from all other objects), the library becomes more comprehensible and easier to develop with.

### Migration guide

Although version 2 brings in a lot of new features, it is generally not backwards-compatible with apps that used version 1 of the library.

This section aims to demonstrate how best to replace v1-compatible code so that your app can work with v2. Each subsection is titled by the feature that is no longer available in v2 and complemented by a v2 workaround.

#### `node.get_instance_type(instance)`
Previously this method returned a string referring to the type of the input instance object. For example, if the instance object is a tell card, then this function would just return 'tell card'.

Instead, the type of the CEInstance can now be directly accessed with `instance.type`, so to get the same result as the deprecated function, one would use `instance.type.name`.

#### `node.get_instance_ce(instance)`
Similarly, this function's effect can now be achieved by accessing a property on the CEInstance directly: `instance.ce`.

#### `node.get_instance_gist(instance)`
Similarly, this function's effect can now be achieved by accessing a property on the CEInstance directly: `instance.gist`.

#### `node.get_concept_ce(concept)`
As above: `concept.ce`.

#### `node.get_concept_gist(concept)`
As above: `concept.gist`.

#### `node.set_agent_name(name)`
Now, onw should access the agent through the node, and update the name directly: `node.agent.set_name(name)`.

#### `node.get_agent_name()`
Similarly, access through the CENode instance: `node.agent.get_name()`.

#### `node.get_instance_value(instance, value_name)` and `node.get_instance_relationship(instance, rel_name)`
It's now possible to achieve the same effect by accessing the property directly. For example, to access the timestamp of a card CEInstance: `card.timestamp` and to get the latest recipient: `card.is_to`.

In these cases, simply lower-case the property name, and replace spaces with underscores. However, if you like to be able to access the value or relationship using the precise label, then you can use `card.property('timestamp')` or `card.property('is to')`.

#### `node.get_instance_values(instance, value_name)` and `node.get_instance_relationships(instance, rel_name)`
Similarly, access through the plural form of the property to get all reported properties of this name: `card.timestamps` and `card.is_tos` (i.e. simply add an 's' onto the end of the property).

As before, you can also use `card.properties('timestamp')` or `card.properties('is to')`. The latter method might be useful if you assign property names with "s"s at the end (e.g. if you want a property called 'sms' and a different one called 'smss').
